#!/usr/bin/env python3
"""
System Monitor
Real-time system resource monitoring tool.
Parameters:
    - interval: Monitoring interval in seconds
    - duration: Total monitoring duration
    - output: Output file for monitoring data
Requires: psutil package
"""
import psutil
import argparse
import logging
from datetime import datetime
import json
import threading
import time
import os
import platform
from collections import deque

class SystemMonitor:
    def __init__(self, interval=1, duration=None, max_points=1000):
        self.interval = interval
        self.duration = duration
        self.max_points = max_points
        self.stop_event = threading.Event()
        self.data = {
            'cpu': deque(maxlen=max_points),
            'memory': deque(maxlen=max_points),
            'disk': deque(maxlen=max_points),
            'network': deque(maxlen=max_points),
            'processes': deque(maxlen=max_points)
        }
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def get_system_info(self):
        return {
            'platform': platform.platform(),
            'processor': platform.processor(),
            'cpu_count': psutil.cpu_count(),
            'memory_total': psutil.virtual_memory().total,
            'disk_partitions': [
                {
                    'device': p.device,
                    'mountpoint': p.mountpoint,
                    'fstype': p.fstype
                } for p in psutil.disk_partitions()
            ]
        }

    def get_cpu_info(self):
        return {
            'percent': psutil.cpu_percent(interval=None, percpu=True),
            'freq': psutil.cpu_freq()._asdict() if psutil.cpu_freq() else None,
            'stats': psutil.cpu_stats()._asdict(),
            'load_avg': psutil.getloadavg()
        }

    def get_memory_info(self):
        vm = psutil.virtual_memory()
        sm = psutil.swap_memory()
        return {
            'virtual': {
                'total': vm.total,
                'available': vm.available,
                'percent': vm.percent,
                'used': vm.used,
                'free': vm.free
            },
            'swap': {
                'total': sm.total,
                'used': sm.used,
                'free': sm.free,
                'percent': sm.percent
            }
        }

    def get_disk_info(self):
        disk_info = {}
        for partition in psutil.disk_partitions():
            try:
                usage = psutil.disk_usage(partition.mountpoint)
                disk_info[partition.device] = {
                    'mountpoint': partition.mountpoint,
                    'fstype': partition.fstype,
                    'total': usage.total,
                    'used': usage.used,
                    'free': usage.free,
                    'percent': usage.percent
                }
            except:
                continue
        return disk_info

    def get_network_info(self):
        net_io = psutil.net_io_counters()
        net_if = psutil.net_if_stats()
        return {
            'io_counters': {
                'bytes_sent': net_io.bytes_sent,
                'bytes_recv': net_io.bytes_recv,
                'packets_sent': net_io.packets_sent,
                'packets_recv': net_io.packets_recv,
                'errin': net_io.errin,
                'errout': net_io.errout,
                'dropin': net_io.dropin,
                'dropout': net_io.dropout
            },
            'interfaces': {
                iface: {
                    'isup': stats.isup,
                    'speed': stats.speed,
                    'mtu': stats.mtu
                } for iface, stats in net_if.items()
            }
        }

    def get_process_info(self):
        processes = []
        for proc in psutil.process_iter(['pid', 'name', 'cpu_percent', 
                                       'memory_percent', 'status']):
            try:
                pinfo = proc.info
                pinfo['cpu_percent'] = proc.cpu_percent()
                processes.append(pinfo)
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue
        return sorted(processes, 
                     key=lambda x: x.get('cpu_percent', 0), 
                     reverse=True)[:10]

    def collect_metrics(self):
        timestamp = datetime.now().isoformat()
        metrics = {
            'timestamp': timestamp,
            'cpu': self.get_cpu_info(),
            'memory': self.get_memory_info(),
            'disk': self.get_disk_info(),
            'network': self.get_network_info(),
            'processes': self.get_process_info()
        }
        
        for key in self.data:
            self.data[key].append(metrics[key])
        
        return metrics

    def monitor(self):
        start_time = time.time()
        system_info = self.get_system_info()
        print("Starting system monitoring...")
        print(f"System: {system_info['platform']}")
        print(f"CPU: {system_info['processor']} ({system_info['cpu_count']} cores)")
        
        try:
            while not self.stop_event.is_set():
                metrics = self.collect_metrics()
                
                print(
                    f"\rCPU: {metrics['cpu']['percent'][0]}% | "
                    f"Memory: {metrics['memory']['virtual']['percent']}% | "
                    f"Disk: {list(metrics['disk'].values())[0]['percent']}%",
                    end='\r'
                )
                
                if self.duration and (time.time() - start_time) >= self.duration:
                    break
                    
                time.sleep(self.interval)
                
        except KeyboardInterrupt:
            pass
        finally:
            print("\nMonitoring stopped")

    def save_results(self, output_file):
        results = {
            'system_info': self.get_system_info(),
            'monitoring_config': {
                'interval': self.interval,
                'duration': self.duration,
                'max_points': self.max_points
            },
            'metrics': {
                key: list(value) for key, value in self.data.items()
            }
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        cpu_data = list(self.data['cpu'])
        mem_data = list(self.data['memory'])
        
        avg_cpu = sum(sum(d['percent'])/len(d['percent']) 
                     for d in cpu_data) / len(cpu_data)
        avg_mem = sum(d['virtual']['percent'] 
                     for d in mem_data) / len(mem_data)
        
        self.logger.info("\nMonitoring Summary:")
        self.logger.info(f"Average CPU Usage: {avg_cpu:.1f}%")
        self.logger.info(f"Average Memory Usage: {avg_mem:.1f}%")
        
        # Find peak resource usage
        peak_cpu = max(max(d['percent']) for d in cpu_data)
        peak_mem = max(d['virtual']['percent'] for d in mem_data)
        
        self.logger.info(f"Peak CPU Usage: {peak_cpu:.1f}%")
        self.logger.info(f"Peak Memory Usage: {peak_mem:.1f}%")

    def start_monitoring(self, output_file=None):
        monitor_thread = threading.Thread(target=self.monitor)
        monitor_thread.start()
        
        try:
            monitor_thread.join()
        except KeyboardInterrupt:
            self.stop_event.set()
            monitor_thread.join()
        
        if output_file:
            self.save_results(output_file)
        
        self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="System Monitor")
    parser.add_argument("--interval", type=float, default=1,
                      help="Monitoring interval in seconds")
    parser.add_argument("--duration", type=int,
                      help="Total monitoring duration in seconds")
    parser.add_argument("--max-points", type=int, default=1000,
                      help="Maximum number of data points to store")
    parser.add_argument("--output", help="Output file for monitoring data")
    args = parser.parse_args()

    monitor = SystemMonitor(args.interval, args.duration, args.max_points)
    monitor.start_monitoring(args.output)
